from edi_835_parser.elements.identifier import Identifier
from edi_835_parser.elements.reference_qualifier import ReferenceQualifier
from edi_835_parser.segments.utilities import split_segment


class Reference:
	identification = 'REF'

	identifier = Identifier()
	qualifier = ReferenceQualifier()

	def __init__(self, segment: str):
		self.segment = segment
		segment = split_segment(segment)

		self.identifier = segment[0]
		self.qualifier = segment[1]
		self.value = segment[2]

	def __repr__(self) -> str:
		return '\n'.join(str(item) for item in self.__dict__.items())

	def __str__(self) -> str:
		return f'{self.qualifier}: {self.value}'


if __name__ == '__main__':
	pass



from typing import Iterator, Tuple, Optional, List
from warnings import warn

from edi_835_parser.segments.claim import Claim as ClaimSegment
from edi_835_parser.segments.entity import Entity as EntitySegment
from edi_835_parser.segments.reference import Reference as ReferenceSegment
from edi_835_parser.segments.date import Date as DateSegment
from edi_835_parser.segments.amount import Amount as AmountSegment
from edi_835_parser.segments.utilities import find_identifier
from edi_835_parser.loops.service import Service as ServiceLoop


class Claim:
	initiating_identifier = ClaimSegment.identification
	terminating_identifiers = [
		ClaimSegment.identification,
		'SE'
	]

	def __init__(
			self,
			claim: ClaimSegment = None,
			entities: List[EntitySegment] = None,
			services: List[ServiceLoop] = None,
			references: List[ReferenceSegment] = None,
			dates: List[DateSegment] = None,
			amount: AmountSegment = None,
	):
		self.claim = claim
		self.entities = entities if entities else []
		self.services = services if services else []
		self.references = references if references else []
		self.dates = dates if dates else []
		self.amount = amount

	def __repr__(self):
		return '\n'.join(str(item) for item in self.__dict__.items())

	@property
	def rendering_provider(self) -> Optional[EntitySegment]:
		rendering_provider = [e for e in self.entities if e.entity == 'rendering provider']
		assert len(rendering_provider) <= 1

		if len(rendering_provider) == 1:
			return rendering_provider[0]

	@property
	def claim_statement_period_start(self) -> Optional[DateSegment]:
		statement_period_start = [d for d in self.dates if d.qualifier == 'claim statement period start']
		assert len(statement_period_start) <= 1

		if len(statement_period_start) == 1:
			return statement_period_start[0]

	@property
	def claim_statement_period_end(self) -> Optional[DateSegment]:
		statement_period_end = [d for d in self.dates if d.qualifier == 'claim statement period end']
		assert len(statement_period_end) <= 1

		if len(statement_period_end) == 1:
			return statement_period_end[0]

	@property
	def patient(self) -> EntitySegment:
		patient = [e for e in self.entities if e.entity == 'patient']
		assert len(patient) == 1

		return patient[0]

	@classmethod
	def build(cls, segment: str, segments: Iterator[str]) -> Tuple['Claim', Optional[Iterator[str]], Optional[str]]:
		claim = Claim()
		claim.claim = ClaimSegment(segment)

		segment = segments.__next__()
		while True:
			try:
				if segment is None:
					segment = segments.__next__()

				identifier = find_identifier(segment)

				if identifier == ServiceLoop.initiating_identifier:
					service, segment, segments = ServiceLoop.build(segment, segments)
					claim.services.append(service)

				elif identifier == EntitySegment.identification:
					entity = EntitySegment(segment)
					claim.entities.append(entity)
					segment = None

				elif identifier == ReferenceSegment.identification:
					reference = ReferenceSegment(segment)
					claim.references.append(reference)
					segment = None

				elif identifier == DateSegment.identification:
					date = DateSegment(segment)
					claim.dates.append(date)
					segment = None

				elif identifier == AmountSegment.identification:
					amount = AmountSegment(segment)
					claim.amount = amount
					segment = None

				elif identifier in cls.terminating_identifiers:
					return claim, segments, segment

				else:
					segment = None
					message = f'Identifier: {identifier} not handled in claim loop.'
					warn(message)

			except StopIteration:
				return claim, None, None



from edi_835_parser.elements import Element

# https://magnacare.com/wp-content/uploads/pdf/MagnacareCompanionGuide_835_5010A1.pdf
entity_types = {
	'1': 'person',
	'2': 'entity',
}


class EntityType(Element):

	def parser(self, value: str) -> str:
		return entity_types.get(value, value)



from typing import Optional, Union

from edi_835_parser.elements import Element


class Integer(Element):

	def parser(self, value: str) -> Optional[Union[int, str]]:
		if value == '':
			return None

		try:
			value = int(value)
		except:
			pass

		return value



from typing import List, Optional


def split_segment(segment: str) -> List[str]:
	"""different payers use different characters to delineate elements"""
	asterisk = '*'
	pipe = '|'

	asterisk_segment_count = len(segment.split(asterisk))
	pipe_segment_count = len(segment.split(pipe))

	if asterisk_segment_count > pipe_segment_count:
		return segment.split(asterisk)
	else:
		return segment.split(pipe)


def find_identifier(segment) -> str:
	segment = split_segment(segment)
	return segment[0]

def get_element(segment: List[str], index: int, default=None) -> Optional[str]:
	element = default
	if index < len(segment):
		element = segment[index]

	return element



from edi_835_parser.elements.authorization_information_qualifier import AuthorizationInformationQualifier
from edi_835_parser.elements.organization import Organization
from edi_835_parser.elements.date import Date
from edi_835_parser.elements.identifier import Identifier
from edi_835_parser.segments.utilities import split_segment


class Interchange:
	identification = 'ISA'

	identifier = Identifier()
	authorization_information_qualifier = AuthorizationInformationQualifier()
	sender = Organization()
	receiver = Organization()
	transmission_date = Date()

	def __init__(self, segment: str):
		self.segment = segment
		segment = split_segment(segment)

		self.identifier = segment[0]
		self.authorization_information_qualifier = segment[1]
		self.sender = segment[6]
		self.receiver = segment[8]
		self.transmission_date = segment[9] + segment[10]

	def __repr__(self):
		return '\n'.join(str(item) for item in self.__dict__.items())


if __name__ == '__main__':
	pass



import os
from typing import List
from warnings import warn

from edi_835_parser.transaction_set.transaction_set import TransactionSet
from edi_835_parser.transaction_set.transaction_sets import TransactionSets


def parse(path: str, debug: bool = False) -> TransactionSets:
	if path[0] == '~':
		path = os.path.expanduser(path)

	transaction_sets = []
	if os.path.isdir(path):
		files = _find_edi_835_files(path)
		for file in files:
			file_path = f'{path}/{file}'
			if debug:
				transaction_set = TransactionSet.build(file_path)
				transaction_sets.append(transaction_set)
			else:
				try:
					transaction_set = TransactionSet.build(file_path)
					transaction_sets.append(transaction_set)
				except Exception as e:
					warn(f'Failed to build a transaction set from {file_path} with error: {e}')
	else:
		transaction_set = TransactionSet.build(path)
		transaction_sets.append(transaction_set)

	return TransactionSets(transaction_sets)


def _find_edi_835_files(path: str) -> List[str]:
	files = []
	for file in os.listdir(path):
		if file.endswith('.txt') or file.endswith('.835'):
			files.append(file)

	return files


def main():
	data = parse('~/Desktop/eobs').to_dataframe()
	data.to_csv('~/Desktop/transaction_sets.csv')


if __name__ == '__main__':
	main()



from edi_835_parser.elements import Element
from edi_835_parser.elements.utilities import split_element


class ServiceQualifer(Element):

	def parser(self, value: str) -> str:
		value = split_element(value)
		qualifier, *_ = value
		return qualifier



from edi_835_parser.elements import Element, Code

# https://x12.org/codes/remittance-advice-remark-codes
remark_codes = {
	'N630': 'Referral not authorized by attending physician.',
	'N650': 'This policy was not in effect for this date of loss. No coverage is available.',
	'M53': 'Missing/incomplete/invalid days or units of service.',
	'M15': 'Separately billed services/tests have been bundled as they are considered components of the same procedure. Separate payment is not allowed.',
	'M80': 'Not covered when performed during the same session/date as a previously processed service for the patient.',
	'M86': 'Service denied because payment already made for same/similar procedure within set time frame.',
	'MA130': 'Your claim contains incomplete and/or invalid information, and no appeal rights are afforded because the claim is unprocessable. Please submit a new claim with the complete/correct information.',
	'N122': 'Add-on code cannot be billed by itself.',
	'N20': 'Service not payable with other service rendered on the same date.',
	'N6': 'Under FEHB law (U.S.C. 8904(b)), we cannot pay more for covered care than the amount Medicare would have allowed if the patient were enrolled in Medicare Part A and/or Medicare Part B.',
	'N640': 'Exceeds number/frequency approved/allowed within time period.',
	'N674': 'Not covered unless a pre-requisite procedure/service has been provided.',
	'N702': 'Decision based on review of previously adjudicated claims or for claims in process for the same/similar type of services.',
	'N781': 'Alert: Patient is a Medicaid/ Qualified Medicare Beneficiary. Review your records for any wrongfully collected deductible. This amount may be billed to a subsequent payer.',
	'N782': 'Alert: Patient is a Medicaid/ Qualified Medicare Beneficiary. Review your records for any wrongfully collected coinsurance. This amount may be billed to a subsequent payer.',
	'N807': 'Payment adjustment based on the Merit-based Incentive Payment System (MIPS).',
}


class RemarkCode(Element):

	def parser(self, value: str) -> Code:
		description = remark_codes.get(value, None)
		return Code(value, description)



from typing import Optional

from edi_835_parser.elements import Element
from edi_835_parser.elements.utilities import split_element


class ServiceModifier(Element):

	def parser(self, value: str) -> Optional[str]:
		value = split_element(value)
		if len(value) > 2:
			return value[2]
